// Line trace
// Create 2012/03/04
// License (CC BY-NC 2.1) 
// Xstech Inc. Mitaka Hardware Engineer Cafe

#define SENSOR_MAX      4  // センサーの数
#define SENSOR_LED      8  // センサー用 LED 番号
#define AVR_COUNT       4  // 集計数 ( 4データを集計し、平均する。)
//#define FULL_POWER

#define SENSOR_PHOTO_1  3  // 位置 LED 1 (PC3)
#define SENSOR_PHOTO_2  2  // 位置 LED 2 (PC2)
#define SENSOR_PHOTO_3  1  // 位置 LED 3 (PC1)
#define SENSOR_PHOTO_4  0  // 位置 LED 4 (PC0)

#define LED1   5  // LED 1 はデジタルピン 5 に接続されている。(PD5)
#define LED2   4  // LED 2 はデジタルピン 4 に接続されている。(PD4)
#define LED3   3  // LED 3 はデジタルピン 3 に接続されている。(PD3)
#define LED4   2  // LED 4 はデジタルピン 2 に接続されている。(PD2)

#define MOTOR_LEFT    9  // PB1
#define MOTOR_RIGHT  10  // PB2

#define SENSOR_BLACK      (600)  // 
#define SENSOR_LOW_LIMIT  (200)  // センサー下限

// 変数
int SensorPhoto[ SENSOR_MAX ] ;
int Sensor[ SENSOR_MAX ][ AVR_COUNT ] ;
int SensorAvr[ SENSOR_MAX ] ;
int AvrNo = 0 ;
int Motor_L, Motor_R ;

//---------------------------------------------------------------
// 初期化
//---------------------------------------------------------------
void setup()
{
  int i, j ;

  pinMode( SENSOR_LED, OUTPUT ) ;     // センサー（発光） LED を ON/OFF する。
  digitalWrite( SENSOR_LED, HIGH ) ;  // センサー LED を ON とする。

  // アナログピンは入力が初期値なので通常は設定する必要はありません。
  // モータ設定
  pinMode( MOTOR_LEFT,  OUTPUT ) ;    // 左モータ アナログ(PWM) 出力用
  pinMode( MOTOR_RIGHT, OUTPUT ) ;    // 右モータ アナログ(PWM) 出力用
 
  // 左モータ方向制御用
  //              +----------------- PB7 出力に設定
  //              |+---------------- PB6
  //              ||
  DDRB = DDRB | 0b11000000 ;
  
  // 右モータ方向制御用
  //                +--------------- PC5 出力に設定
  //                |+-------------- PC4
  //                ||
  DDRC = DDRC | 0b00110000 ;

  // 変数クリアー
  for ( i = 0 ; i < SENSOR_MAX ; i++ ) {
    SensorPhoto[ i ] = 0 ;
    for ( j = 0 ; j < AVR_COUNT ; j++ ) {
      Sensor[ i ][ j ] = 0 ; 
    }
  }

  // テスト用
  pinMode( LED1, OUTPUT ) ;  // LED 1 (デジタルピン 5 ) を出力に設定する。
  pinMode( LED2, OUTPUT ) ;  // LED 2 (デジタルピン 4 ) を出力に設定する。
  pinMode( LED3, OUTPUT ) ;  // LED 3 (デジタルピン 3 ) を出力に設定する。
  pinMode( LED4, OUTPUT ) ;  // LED 4 (デジタルピン 2 ) を出力に設定する。

  // LED すべて消灯する。
  digitalWrite( LED1, HIGH ) ;  // LED 1 消灯
  digitalWrite( LED2, HIGH ) ;  // LED 2 消灯
  digitalWrite( LED3, HIGH ) ;  // LED 3 消灯
  digitalWrite( LED4, HIGH ) ;  // LED 4 消灯
  // 動作確認用
  digitalWrite( LED1, LOW ) ;   // LED 1 点灯
}

//---------------------------------------------------------------
//  メイン
//---------------------------------------------------------------
void loop()
{
  int i ;
  int c ;

  GetSensor() ;
  // 平均値を求める。集計値 ÷ 4 (小数点以下は切り捨て）
  Motor_R = SensorAvr[1] / AVR_COUNT ;
  Motor_L = SensorAvr[2] / AVR_COUNT ;

  if ( Motor_L <= SENSOR_LOW_LIMIT || Motor_R <= SENSOR_LOW_LIMIT ) {
    StopMotor() ;
    digitalWrite( LED2, HIGH ) ;  // LED 2 消灯
    digitalWrite( LED3, HIGH ) ;  // LED 3 消灯
  } else {
    // 両方紙の上（前進）
    c = AvrNo - 1 ;
    if ( c < 0 ) c = AVR_COUNT - 1 ;  // 最新データ位置
    
    if ( Sensor[ 1 ][ c ] <= SENSOR_BLACK ) {
      Motor_R = Sensor[ 1 ][ c ] / 4 ;  // 右カーブ対策
    }
    if ( Sensor[ 2 ][ c ] <= SENSOR_BLACK ) {
      Motor_L = Sensor[ 2 ][ c ] / 4 ;  // 左カーブ対策
    }

#ifdef FULL_POWER
    Motor_L = Motor_L * 2 / 3 ;
    Motor_R = Motor_R * 2 / 3 ;
#endif
    Motor_L >>= 2 ;  // アナログ出力は 0～255 までなのでセンサー値を1/4にする。
    Motor_R >>= 2 ; 

    ForwardMotor() ;
    analogWrite( MOTOR_RIGHT, Motor_R ) ;
    analogWrite( MOTOR_LEFT,  Motor_L ) ;
    digitalWrite( LED2, LOW ) ;  // LED 2 点灯
    digitalWrite( LED3, LOW ) ;  // LED 3 点灯
  }
  delay( 2 ) ;
}

//---------------------------------------------------------------
// Photo diod センサーの読み込み
//---------------------------------------------------------------
void GetSensor( void )
{
  int i ;

  // センサー値を読み込む
  SensorPhoto[0] = analogRead( SENSOR_PHOTO_1 ) ;
  SensorPhoto[1] = analogRead( SENSOR_PHOTO_2 ) ;  // 右センサー
  SensorPhoto[2] = analogRead( SENSOR_PHOTO_3 ) ;  // 左センサー
  SensorPhoto[3] = analogRead( SENSOR_PHOTO_4 ) ;

  // 移動平均用にデータを保存する。
  for ( i = 0 ; i < SENSOR_MAX ; i++ ) {
    SensorAvr[ i ] -= Sensor[ i ][ AvrNo ] ;  // 前回の値を減算する。
    Sensor[ i ][ AvrNo ] = SensorPhoto[ i ] ; // 今回の値を保存する。（次、減算用)
    SensorAvr[ i ] += Sensor[ i ][ AvrNo ] ;  // 今回の値を加算する。
  }
  if ( ++AvrNo >= AVR_COUNT ) {
    AvrNo = 0 ;
  }
}

/* モーター方向設定
 Left Motor            Right Motor
   PB7 PB6              PC5 PC4
   IN2 IN1 ( A )        IN2 IN1 ( B ) 
    0   0  Stop          0   0  Stop
    0   1  Foward        0   1  Foward
    1   0  Reveser       1   0  Reveser
    1   1  Break         1   1  Break
*/
//---------------------------------------------------------------
// Motor 左右停止
//---------------------------------------------------------------
void StopMotor( void )
{
  // 左モータを停止する。
  //                +----------------- PB7
  //                |+---------------- PB6
  //                ||
  PORTB = PORTB & 0b00111111 ;
    
  // 右モータを停止する。
  //                  +--------------- PC5
  //                  |+-------------- PC4
  //                  ||
  PORTC = PORTC & 0b11001111 ;
}

//---------------------------------------------------------------
// Motor 前進
//---------------------------------------------------------------
void ForwardMotor( void )
{
  // 左モータ前進。
  //                  +------------------ PB7 --> 0
  //                  |+----------------- PB6 --> 0
  //                  ||              +-- PB6 --> 1
  //                  ||              |
  PORTB = ( PORTB & 0b00111111 ) | 0b01000000 ;
    
  // 右モータ前進。
  //                    +------------------ PC5 --> 0
  //                    |+----------------- PC4 --> 0
  //                    ||              +-- PC4 --> 1
  //                    ||              |
  PORTC = ( PORTC & 0b11001111 ) | 0b00010000 ;
}

